#python

import lx
from farfarer_vnorm import vnormkit

# Morph map check variables.
morphMapID = -1
haveMorphMap = False
# Get the first selected morph map.
selectedMorphMapName = lx.eval1('vertMap.list type:morf ?')
if selectedMorphMapName != '_____n_o_n_e_____':
	morphMaps = lx.evalN('query layerservice vmaps ? morph')
	for morphMap in morphMaps:
		morphMapName = lx.eval1('query layerservice vmap.name ? %s' % morphMap)
		if morphMapName == selectedMorphMapName:
			morphMapID = morphMap
			haveMorphMap = True
			break

# If there still wasn't one, then end the script.
if haveMorphMap:

	selectionType = vnormkit.getSelectionType()
	selRestore = vnormkit.saveSelection()
	vnormkit.renameNormalMapsDefault()

	# Get the build number (701 SP1 check).
	slowVersion = vnormkit.isSP1()

	# Rename the normal maps to 'vert_normals'.
	vnormkit.renameNormalMapsDefault()

	mainLayer = lx.eval1('query layerservice layers ? main')
	mainLayerID = lx.eval1('query layerservice layer.id ? %s' % mainLayer)
	layers = lx.evalN('query layerservice layers ? fg')

	for layer in layers:
		# Select each foreground layer individually, helps stop issues with vertex normal maps.
		layerID = lx.eval1('query layerservice layer.id ? %s' % layer)
		lx.eval('select.subItem %s set mesh' % layerID)

		# Bake the normals. Just to be sure the map exists.
		if vnormkit.doesNormalMapExist(vnormkit.defaultNormalName):
			lx.eval('!!select.vertexMap %s norm 3' % vnormkit.defaultNormalName)
		lx.eval('vertMap.normals')
		# Select the normal map.
		lx.eval('select.vertexMap %s norm replace' % vnormkit.defaultNormalName)

		vertices = lx.evalN('query layerservice verts ? all')
		#lx.out('Vertices: %s' % vertices)
		morphID = lx.eval1('query layerservice vmap.index ? %s' % morphMapID)
		for vert in vertices:
			# Grab morph value for vertex.
			morphValue = lx.evalN('query layerservice vert.vmapValue ? %s' % vert)
			#lx.out('Morph value: %s' % morphValue)

			# Check to see if it has a morph value, if not we'll use vertex normal.
			if len(morphValue) == 0:
				morphValue = lx.evalN('query layerservice vert.normal ? %s' % vert)
			else:
				# Check to see if the morph is zero length (i.e. the same as it's unmorphed position).
				# If it is, grab and use the vertex normal instead.
				# If it isn't, then normalize it and use that.
				length = vnormkit.vector_length(morphValue)
				if length == 0.0:
					morphValue = lx.evalN('query layerservice vert.normal ? %s' % vert)
				else:
					morphValue = vnormkit.vector_normalize_length(morphValue, length)
				

			if slowVersion:
				lx.eval('select.element %s vertex set %s' % (layer, vert))
				lx.eval('vertMap.setValue normal 0 %s' % morphValue[0])
				lx.eval('vertMap.setValue normal 1 %s' % morphValue[1])
				lx.eval('vertMap.setValue normal 2 %s' % morphValue[2])
			else:
				lx.eval('vertMap.setVertex %s normal 0 %s %s' % (vnormkit.defaultNormalName, vert, morphValue[0]))
				lx.eval('vertMap.setVertex %s normal 1 %s %s' % (vnormkit.defaultNormalName, vert, morphValue[1]))
				lx.eval('vertMap.setVertex %s normal 2 %s %s' % (vnormkit.defaultNormalName, vert, morphValue[2]))

			if slowVersion:
				lx.eval('select.drop vertex')

	# Restore original layer selection.
	lx.eval('select.subItem %s set mesh' % mainLayerID)
	for layer in layers:
		layerID = lx.eval1('query layerservice layer.id ? %s' % layer)
		lx.eval('select.subItem %s add mesh' % layerID)

	vnormkit.renameNormalMapsCustom(True)

	# Deselect morph map.
	lx.eval('select.vertexMap %s morf 3' % morphMapName)

	vnormkit.restoreSelection(selRestore)
	lx.eval('select.type %s' % selectionType)

else:
	lx.eval('dialog.setup error')
	lx.eval('dialog.title {Failed to find morph map.}')
	lx.eval('dialog.msg {You must have a relative morph map selected.}')
	lx.eval('dialog.result ok')
	lx.eval('dialog.open')